function gradient_v104(fname1,fname_disp_1,clri1,fname2,fname_disp_2,clri2,num_cells_desired,thresholdA,thresholdB,cutoff_brightness,min_brightness,min_region_size,max_region_size,max_radius,num_frames,frame_rate)

%filename.m, A MATLAB script to automatically analyse fluorescent imaging data, Copyright (C) 2013 Peter A. Appleby

%This program is free software: you can redistribute it and/or modify it under the terms 
%of the GNU General Public License as published by the Free Software Foundation version 3 
%of the License. See the GNU General Public License for more details.

% fname1,2 - image to load and analyse
% fname_disp_1,2 - image to display ROIs over
% clri1,2 - colour channel to read, red=1, green=2, blue=3
% num_cells_desired - target number of ROIs we want to extract
% thresholdA - higher threshold, for identifying a cell ROI
% thresholdB - lower threshold, for identifying the halo region around each ROI
% cutoff_brightness - the brightness threshold used when the image is masked during pre-processing
% min_brightness - the minimum brightness that we class as being 'of interest'
% min_region_size - the minimum ROI size
% max_region_size - the maximum ROI size
% max_radius - the maximum radius that the ROI must fit into completely
% num_frames - number of frames to examine
% frame_rate - the frame rate of the recorded images

% -------------------------------------------------------------------------
% Part I - Combine the specified red and green images
% -------------------------------------------------------------------------

% Read in the first image, find its size, and create an image ID number
im1=imread(fname1);im1_size=size(im1);id=1;

% Display the first image
figure(id);image(im1);id=id+1;

% Check if there is a second image specified
if fname2=='-'  
    
    % There is no second image, so im2 is just set to be a copy of the first image
    im2=imread(fname1);
    
else    
    % There is a second image, so read it in
    im2=imread(fname2); 
end    
   
% Produce a summed image. 
im=zeros(im1_size(1),im1_size(2),3);min_value=0;max_value=0;    

% Take the mean of each pixel, and record the min and max pixel values
for y=1:im1_size(1)
    for x=1:im1_size(2)
        im(y,x,1)=(double(im1(y,x,clri1))+double(im2(y,x,clri2)))/255.0;
        if im(y,x,1)>=max_value
            max_value=im(y,x,1);
        end
        if im(y,x,1)<=min_value
            min_value=im(y,x,1);
        end            
    end
end    

% Make the image magenta by setting the B channel to the same value as the R channel 
for y=1:im1_size(1)
    for x=1:im1_size(2)           
        im(y,x,2)=0;
        im(y,x,3)=im(y,x,1);
    end
end

% -------------------------------------------------------------------------
% Part II - Carry out the ROI extraction
% -------------------------------------------------------------------------

% Change the description of the image so it is (x,y) and store it in image[]
im_size=size(im);image_calc=zeros(im_size(2),im_size(1),3);
for y=1:im_size(1)
    for x=1:im_size(2)               

        image_calc(x,y,1)=im(y,x,1);
        image_calc(x,y,2)=im(y,x,2);
        image_calc(x,y,3)=im(y,x,3);
        
    end
end

% Pre-processing step 1: mask the image by setting all pixels with a brightness < min_brightness to zero 
for y=1:im_size(1)
    for x=1:im_size(2)
        if image_calc(x,y,1) < cutoff_brightness
            
            % Remove this pixel from image_calc
            image_calc(x,y,1)=0;
            image_calc(x,y,2)=0;
            image_calc(x,y,3)=0;
            
            % Remove this pixel from from im
            im(y,x,1)=0;
            im(y,x,2)=0;
            im(y,x,3)=0;

        end
    end
end

% Pre-processing step 2: mask the image by setting all pixels with a brightness gradient of zero, to zero 
num_preprocessing_its=0
for a=1:num_preprocessing_its
	mean_brightness_calc=zeros(im_size(2),im_size(1),3);av_size=12;brightness_modifier=0.98; 
	for y=1:im_size(1)
	    for x=1:im_size(2)
			mean_brightness=0;
			% If we are not on a border pixel
			if y>av_size & x> av_size & y<im_size(1)-av_size-1 & x<im_size(2)-av_size-1
				% Calculate the mean brightness of the area			
				for x_1=0:av_size*2
    	        	for y_1=0:av_size*2
    	            	% Locate the current pixel
    	                x_current=x-av_size+x_1;
    	                y_current=y-av_size+y_1;
						% Update the mean brightness
						mean_brightness=mean_brightness+im(y_current,x_current)/((av_size*2+1)^2);
					end
				end
			end
			% Update the mean brightness image
			mean_brightness_calc(y,x,1)=mean_brightness;
			mean_brightness_calc(y,x,2)=0;
			mean_brightness_calc(y,x,3)=mean_brightness;
    	end
	end
	
	for y=1:im_size(1)
    	for x=1:im_size(2)
			if im(y,x,1)<mean_brightness_calc(y,x,1)*brightness_modifier
				mean_brightness_calc(y,x,1)=0;	
				mean_brightness_calc(y,x,3)=0;
			end
		end
	end
	for y=1:im_size(1)
    	for x=1:im_size(2)
			im(y,x,1)=mean_brightness_calc(y,x,1);
			im(y,x,2)=mean_brightness_calc(y,x,2);
			im(y,x,3)=mean_brightness_calc(y,x,3);
			image_calc(x,y,1)=mean_brightness_calc(y,x,1);
			image_calc(x,y,2)=mean_brightness_calc(y,x,2);
			image_calc(x,y,3)=mean_brightness_calc(y,x,3);
		end
	end

end

% Create the container that will be used for storing pixels
pixels=zeros(im_size(2)*im_size(1),5);num_pixels=1;

% Make a special matrix that records whether pixels have been included in the pixels array
included_image=zeros(im_size(2),im_size(1),2);
for y=1:im_size(1)
    for x=1:im_size(2)
        % A zero means that the pixel has NOT YET been added to the pixel array
        included_image(x,y,1)=0;
        % A zero means that the pixel is a halo pixel, and a 1 means it is a region pixel
        included_image(x,y,2)=0;
    end
end

% Create the container that lets us know where the regions change
regions=zeros(num_cells_desired);num_regions=1;regions(1)=2;num_rejected=0;

% Now iterate through the algorithm to find regions
for its=1:num_cells_desired
        
    its
        
    % Choose the top left pixel as the start pixel
    x_chosen=1;
    y_chosen=1;
    brightness=image_calc(x_chosen,y_chosen,1);
        
    % Go through the entire image and identify the brightest pixel
    for y_coord=1:im_size(1)
        for x_coord=1:im_size(2)
            % Check if this pixel is brighter than the current chosen pixel
            if image_calc(x_coord,y_coord,1)>brightness
                % It is, so make this the current pixel
                x_chosen=x_coord;
                y_chosen=y_coord;
                brightness=image_calc(x_chosen,y_chosen,1);
            end
        end
    end        
    
    % Halt if there are no more pixels above the minimum brightness threshold
    if brightness<min_brightness
        disp('No more pixels above minimum brightness, halting')
        break
    end
    
    % Set the threshold - relative values
    threshold_brightness=brightness-brightness*thresholdA;
    threshold_brightness_halo=brightness-brightness*thresholdB;
    threshold_brightness_halo_moving=brightness-brightness*thresholdA;
        
    % Add the first pixel
    num_pixels=num_pixels+1;
    pixels(num_pixels,1)=x_chosen;   % x co-ord
    pixels(num_pixels,2)=y_chosen;   % y co-ord
    pixels(num_pixels,3)=0;          % has this pixel's neighbours been examined already
    pixels(num_pixels,4)=1;          % Is this a pixel we include in our calculations, or a halo pixel
    pixels(num_pixels,5)=0;
    
    num_regions=num_regions+1;          % Note that we have an additional region now
    regions(num_regions)=num_pixels;    % Record that the pixels that make it up start at pixels(num_pixels,?)
    
    included_image(x_chosen,y_chosen,1)=1; % Record this pixel as having been already included    
    
    % Make sure code knows there is at least 1 valued pixel (the start pixel)
    found_new_pixels=1;    
    
    while found_new_pixels==1
        
        % Set the found new pixels flag to zero
        found_new_pixels=0;
        
        % Go through each pixel in the pixels list and check it's neighbours
        for p=1:num_pixels
            % Has this pixel been examined yet
            if pixels(p,3)==0
                % In a 3x3 block around the centre pixel
                for x=0:2
                    for y=0:2
                        % Locate the current pixel
                        x_current=pixels(p,1)-1+x;
                        y_current=pixels(p,2)-1+y;
                        
                        % Is this a valid location in terms of box size?
                        if x_current>0 & x_current<=im_size(2) & y_current>0 & y_current<=im_size(1)
                            
                            % Check to see if this pixel is bright enough to at least be in the halo                             
                            if image_calc(x_current,y_current,1)>threshold_brightness_halo
                                
                                % It is, so check to see if it has already been included in the pixels container                                
                                is_this_a_new_pixel=0;
                                if included_image(x_current,y_current,1)==0
                                    is_this_a_new_pixel=1;
                                end
                                
                                % If it is new, add it to the array
                                if is_this_a_new_pixel==1
                                    
                                    % Add the pixel to the array
                                    num_pixels=num_pixels+1;
                                    pixels(num_pixels,1)=x_current;
                                    pixels(num_pixels,2)=y_current;
                                    pixels(num_pixels,3)=0;
                                    
                                    % Is the pixel bright enough to be
                                    % part of the region AND is being
                                    % checked from a pixel that is
                                    % itself part of a region 
                                    if image_calc(x_current,y_current,1)>threshold_brightness & pixels(p,4)==1 
                                        
                                        % Is the pixel a valid location in terms of distance from start pixel?
                                        if sqrt( (x_current-x_chosen)^2 + (y_current-y_chosen)^2 ) < max_radius                                            
                                            pixels(num_pixels,4)=1;
                                            included_image(x_current,y_current,2)=1;                                            
                                        end
                                    else
                                        
                                        % Update the current halo theshold
                                        if image_calc(x_current,y_current,1)<threshold_brightness_halo_moving
                                            threshold_brightness_halo_moving=image_calc(x_current,y_current,1);
                                        end
                                        
                                        % If it is ALSO within 10% of the
                                        % current halo threshold, add it as
                                        % a halo pixel                                        
                                        if image_calc(x_current,y_current,1) < threshold_brightness_halo_moving
                                            pixels(num_pixels,4)=0;
                                            included_image(x_current,y_current,2)=0;
                                        end
                                            
                                    end
                                    found_new_pixels=1;
                                    regions(num_regions)=regions(num_regions)+1;
                                    
                                    % Record this pixel as now having been included
                                    included_image(x_current,y_current,1)=1; 
                                end
                            end
                           
                        end
                    end
                end
                % This pixel's neighbours have now been checked, so note that in the log
                pixels(p,3)=1;
            end
        end
        
    end
    
    % Check the region is large enough
    num_pixels_in_region=0;        
    for p=regions(num_regions-1):regions(num_regions)
        if pixels(p,4)==1
            num_pixels_in_region=num_pixels_in_region+1;
        end
    end
    
    if num_pixels_in_region<min_region_size || num_pixels_in_region>max_region_size
        % It's not big enough, so make all the pixels in this region halo pixels     
        for p=regions(num_regions-1):regions(num_regions)
            pixels(p,4)=0;
        end        
        if num_pixels_in_region<min_region_size
			disp('Rejected region - too small')
		end
		if num_pixels_in_region>max_region_size
			disp('Rejected region - too large')
		end
        num_rejected=num_rejected+1;
    else
        %disp('Accepted region')
    end
    
    % Calculate which pixels are outline pixels
    for p=regions(num_regions-1):regions(num_regions)
        if pixels(p,4)==1
            % It's a region pixel, so check that it has at least one halo
            % neighbour in a 3x3 block around the centre pixel
            for x=0:2
                for y=0:2
                    % Locate the current pixel
                    x_current=pixels(p,1)-1+x;
                    y_current=pixels(p,2)-1+y;
                    % Is this a valid location?
                    if x_current>0 & x_current<=im_size(2) & y_current>0 & y_current<=im_size(1)                        
                        % Is the examined pixel a halo pixel? If so, the
                        % current pixel is an outline pixel
                        if included_image(x_current,y_current,2)==0                        
                            pixels(p,5)=1;                        
                        end
                    end
                end
            end
        end
    end
    
    % Fill in the regions, blue for region/outline black for halo
    for p=regions(num_regions-1):regions(num_regions)
        if pixels(p,4)==1
            im(pixels(p,2),pixels(p,1),1)=0;
            im(pixels(p,2),pixels(p,1),2)=0;
            im(pixels(p,2),pixels(p,1),3)=255;
        else
            im(pixels(p,2),pixels(p,1),1)=0;
            im(pixels(p,2),pixels(p,1),2)=0;
            im(pixels(p,2),pixels(p,1),3)=0;
        end
    end
    
    % Now remove the entire region
    for p=regions(num_regions-1):regions(num_regions)
        image_calc(pixels(p,1),pixels(p,2),1)=0;
        im(pixels(p,2),pixels(p,1),1)=0;
        im(pixels(p,2),pixels(p,1),2)=0;
        im(pixels(p,2),pixels(p,1),3)=0;
    end
    
    figure(id);image(im);
    
    pause(0.01);
end

disp('Found ');num_cells_desired-num_rejected

% -------------------------------------------------------------------------
% Part III - Display the ROIs and delete any as required
% -------------------------------------------------------------------------

% Reload the original image and display the contours 
im1=imread(fname_disp_1);id=id+1;
if fname_disp_2=='-'
    im=im1;
else
    im2=imread(fname_disp_2);
    im=im1+im2;    
end

% Fill in the outlines
for n=2:num_regions
    for p=regions(n-1):regions(n)
        if pixels(p,5)==1
			for x=0:2
                for y=0:2
                    % Locate the current pixel
                    x_current=pixels(p,1)-1+x;
                    y_current=pixels(p,2)-1+y;
            		
					% Is this a valid location?
                    if x_current>0 & x_current<=im_size(2) & y_current>0 & y_current<=im_size(1) 
						im(y_current,x_current,1)=255;
	            		im(y_current,x_current,2)=255;
    			        im(y_current,x_current,3)=255;    
					end    
				end
			end
        end
    end
end

% Add the labels if required
for r=1:num_regions-1
	
    if r==1
        txt='1';
        pos = [pixels(2,1),pixels(2,2)];
        h = text(pos(1),pos(2),txt,'Color',[1 1 1]);
			
	end

	if r>1	
        txt=int2str(r);
        pos = [pixels(regions(r)+1,1),pixels(regions(r)+1,2)];
        h = text(pos(1),pos(2),txt,'Color',[1 1 1]);
	end
end

% Display the image
figure(id);imagesc(im);id=id+1;

x=input('Enter an integer ROI id to tag ROI for deletion, enter -1 to end:');

while x>0
    fprintf('\tDeleting ROI %g\n',x)
    
    % Make all the pixels in this region halo pixels     
    for p=regions(x):regions(x+1)
        pixels(p,4)=0;
        pixels(p,5)=0;
    end
    
    x=input('Enter an integer ROI id to tag ROI for deletion, enter -1 to end:');
end
   if x<0
        disp('Finished')
   end

% -------------------------------------------------------------------------
% Part IV - Show the final set of ROIs
% -------------------------------------------------------------------------

% Reload the original image and display the contours 
im1=imread(fname_disp_1);
if fname_disp_2=='-'
    im=im1;
else
    im2=imread(fname_disp_2);
    im=im1+im2;    
end

% Fill in the outlines
for n=2:num_regions
    for p=regions(n-1):regions(n)
        if pixels(p,5)==1
			for x=0:2
                for y=0:2
                    % Locate the current pixel
                    x_current=pixels(p,1)-1+x;
                    y_current=pixels(p,2)-1+y;
            		
					% Is this a valid location?
                    if x_current>0 & x_current<=im_size(2) & y_current>0 & y_current<=im_size(1) 
						im(y_current,x_current,1)=255;
	            		im(y_current,x_current,2)=255;
    			        im(y_current,x_current,3)=255;    
					end    
				end
			end
        end
    end
end

% Add the labels if required
for r=1:num_regions-1
	
    if r==1
        txt='1';
        pos = [pixels(2,1),pixels(2,2)];
        h = text(pos(1),pos(2),txt,'Color',[1 1 1]);
			
	end

	if r>1	
        txt=int2str(r);
        pos = [pixels(regions(r)+1,1),pixels(regions(r)+1,2)];
        h = text(pos(1),pos(2),txt,'Color',[1 1 1]);
	end
end

% Display the image
figure(id);imagesc(im);id=id+1;
   
% -------------------------------------------------------------------------
% Part V - Extract intensity time sequences
% -------------------------------------------------------------------------

% Create a zeroed array holding the time evolution of the mean intensity values
% of each identified region
mean_intensities=zeros(num_regions,num_frames,1);


% Loop through the frames and update the mean_intensities for each region
for frame=1:num_frames
    
    if frame<=10
        frame_name = sprintf('Slide1a x10 Wound Ratio 000%d.jpg', frame-1)
    else
        if frame<=100
            frame_name = sprintf('Slide1a x10 Wound Ratio 00%d.jpg', frame-1)
        else            
            if frame<=1000
                frame_name = sprintf('Slide1a x10 Wound Ratio 0%d.jpg', frame-1)
            else            
                if frame<=10000
                    frame_name = sprintf('Slide1a x10 Wound Ratio %d.jpg', frame-1)
                end
            end
        end
    end
    
    % Load the current frames
    current_frame=imread(frame_name);  
        
    mean_intensities(1,frame,1)=frame;
    
    % Loop through the regions
    for region=2:num_regions
        calc_intensity=0;
        calc_pixels=0;
        % Loop through all pixels in this region
        for p=regions(region-1):regions(region)
            % Is this a region pixel?
            if pixels(p,4)==1
                % If we are dealing with a ratioed image, just take the intensity
	            calc_intensity=calc_intensity+double(current_frame(pixels(p,2),pixels(p,1),2));               
                
				% In all cases, we've added another pixel to the mean of the region
				calc_pixels=calc_pixels+1;

            end
        end
        mean_intensities(region,frame,1)=calc_intensity/calc_pixels;
    end
    pause(0.01);
end

location_array=zeros(1);
index=1;
output_code5='';
% Loop through the regions
for region=2:num_regions
    % Find the start pixel of the current region
    p=regions(region);
    % Check if the start pixel is part of a region
    % Loop through all pixels in this region
    for p=regions(region-1):regions(region)   
        if pixels(p,4)==1
            % Extract the ID and x-coord of this start pixel
            %location_array(index,1)=region-1;
            location_array(index,1)=pixels(p,1);
            output_code5 =  strcat(output_code5,'%f \t');    
            index=index+1;
            break;
        end
    end
end
output_code5 =  strcat(output_code5,'\n');

location_array

file = fopen('Data','w');

clipped_array=mean_intensities(2:end, :);

% Convert frame number to real time using the frame rate parameter
for frame=1:num_frames
    mean_intensities(1,frame,1)=(mean_intensities(1,frame,1)/frame_rate);
end

% Create the output codes
output_code2='';
for r=2:num_regions
    output_code2 =  strcat(output_code2,'%f \t');    
end
output_code2 =  strcat(output_code2,'\n');

output_code1='';
for r=1:num_regions-1
    output_code1 =  strcat(output_code1,'%i \t');    
end
output_code1 =  strcat(output_code1,'\n');

output_code3='%s \t';

output_code4='';
for r=1:num_regions
    output_code4 =  strcat(output_code4,'%f \t');    
end
output_code4 =  strcat(output_code4,'\n');

fprintf(file, output_code2, clipped_array);

load Data

plot(Data)

title('ROI intensities');
xlabel('Frame');
ylabel('Intensity');

file2 = fopen('Data_labelled','w');

labels=zeros(1,num_regions-1);
for r=1:num_regions-1
    labels(1,r) = r;
end

fprintf(file2, output_code3, 'Time');
fprintf(file2, output_code1, labels);
fprintf(file2, output_code4, mean_intensities);

% Finally, output data on ROI location
file3 = fopen('ROI_locations','w');

labels=zeros(1,num_regions-1);
for r=1:num_regions-1
    labels(1,r) = r;
end

fprintf(file3, output_code5, location_array(2));
